<?php

// Prevent data leaks.
defined('ABSPATH') || exit;

class WC_Conectenvios
{
  public function __construct()
  {
    if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
      return;
    }

    $this->init_admin();
    $this->hooks();
  }

  public function init_admin()
  {
    include_once WCCE_PLUGIN_PATH . 'includes/admin/class-wcce-admin.php';
    new WCCE_Admin();
  }

  public function hooks()
  {
    add_action('woocommerce_shipping_init', [$this, 'includes']);
    add_filter('woocommerce_shipping_methods', [$this, 'include_wcce_shipping_methods']);

    if ('yes' === get_option('wcce_options')['wcce_field_generate_tag']) {
      add_action('woocommerce_thankyou', [$this, 'create_cart'], 10, 1);
      add_filter('woocommerce_checkout_fields', [$this, 'custom_override_checkout_fields']);
      add_action('woocommerce_checkout_update_order_meta', [$this, 'save_custom_checkout_fields']);
      add_action('woocommerce_checkout_process', [$this, 'custom_validate_billing_phone']);
    }
  }

  public function includes()
  {
    include_once WCCE_PLUGIN_PATH . 'includes/class-wcce-api.php';
    include_once WCCE_PLUGIN_PATH . 'includes/class-wcce-calculator.php';
    include_once WCCE_PLUGIN_PATH . 'includes/shipping/class-wcce-shipping-method.php';
    include_once WCCE_PLUGIN_PATH . 'includes/shipping/class-wcce-shipping-method-correios-mini.php';
    include_once WCCE_PLUGIN_PATH . 'includes/shipping/class-wcce-shipping-method-correios-pac.php';
    include_once WCCE_PLUGIN_PATH . 'includes/shipping/class-wcce-shipping-method-correios-sedex.php';
    include_once WCCE_PLUGIN_PATH . 'includes/shipping/class-wcce-shipping-method-jadlog-dotcom.php';
    include_once WCCE_PLUGIN_PATH . 'includes/shipping/class-wcce-shipping-method-jadlog-package.php';
    include_once WCCE_PLUGIN_PATH . 'includes/shipping/class-wcce-shipping-delivery-forecast.php';
  }

  public function include_wcce_shipping_methods($methods = array())
  {
    $methods['wcce-correios-mini']  = 'WCCE_Shipping_Method_Correios_MINI';
    $methods['wcce-correios-pac']   = 'WCCE_Shipping_Method_Correios_PAC';
    $methods['wcce-correios-sedex'] = 'WCCE_Shipping_Method_Correios_SEDEX';
    $methods['wcce-jadlog-dotcom']  = 'WCCE_Shipping_Method_Jadlog_DOTCOM';
    $methods['wcce-jadlog-package'] = 'WCCE_Shipping_Method_Jadlog_PACKAGE';
    return $methods;
  }

  public function save_custom_checkout_fields( $order_id) 
  {
		if (wp_verify_nonce(!empty($_POST['billing_document']))) {
			update_post_meta($order_id, 'billing_document', sanitize_text_field($_POST['billing_document']));
		}
		if (wp_verify_nonce(!empty($_POST['billing_address_number']))) {
			update_post_meta($order_id, 'billing_address_number', sanitize_text_field($_POST['billing_address_number']));
		}
		if (wp_verify_nonce(!empty($_POST['billing_neighborhood']))) {
			update_post_meta($order_id, 'billing_neighborhood', sanitize_text_field($_POST['billing_neighborhood']));
		}
	}

  public function custom_validate_billing_phone() {
      $is_correct = preg_match('/^[0-9]{11,20}$/', $_POST['billing_phone']);
      if ( $_POST['billing_phone'] && !$is_correct) {
          wc_add_notice( __( 'O número de telefone deve conter de 11 a 20 caracteres, formado por DDD+Número' ), 'error' );
      }
  }

  public function custom_override_checkout_fields( $fields) 
  {
		$fields['billing']['billing_document'] = array(
			'type'		  => 'number',
			'label'       => __('Documento', 'woocommerce-conectenvios'),
			'placeholder' => __('Seu CPF ou RG', 'woocommerce-conectenvios'),
			'placeholder' => '',
			'required'    => true,
			'class'       => array('form-row-wide'),
			'clear'       => true,
			'priority'    => 25
		);

		$fields['billing']['billing_address_number'] = array(
			'type'		  => 'number',
			'label'       => __('Número', 'woocommerce-conectenvios'),
			'placeholder' => '',
			'required'    => true,
			'class'       => array('form-row-wide'),
			'clear'       => true,
			'priority'    => 64
		);

		$fields['billing']['billing_neighborhood'] = array(
			'label'       => __('Bairro', 'woocommerce-conectenvios'),
			'placeholder' => '',
			'required'    => true,
			'class'       => array('form-row-wide'),
			'clear'       => true,
			'priority'    => 65
		);

		return $fields;
	}

  public function create_cart( $order_id )
  {
    if (!$order_id) {
			return;
		}

    $token        = get_option('wcce_options')['wcce_field_token'];
    $api          = new WCCE_Api($token);
    $calc         = new WCCE_Calculator();

    $order           = wc_get_order($order_id);
    $cart_name       = '#' . $order_id . ' ' . $order->get_formatted_billing_full_name();
    $measurements    = $calc->calculate_order_measurements($order_id);
    $shipping_method = explode(' ', WC()->session->get('shipping_for_package_0')['rates'][WC()->session->get('chosen_shipping_methods')[0]]->label);
    $postal          = [ 'service_name' => strtoupper( str_replace(['(', ')'], '', $shipping_method[1]) ) ];

    $salesman = [
      'name'     => get_option('wcce_options')['wcce_field_salesman_name'],
      'phone'    => get_option('wcce_options')['wcce_field_salesman_phone'],
      'document' => get_option('wcce_options')['wcce_field_salesman_document'],
      'address'  => [
        'street'       => get_option('wcce_options')['wcce_field_salesman_address'],
        'number'       => get_option('wcce_options')['wcce_field_salesman_address_number'],
        'zipcode'      => get_option('wcce_options')['wcce_field_salesman_address_zipcode'],
        'neighborhood' => get_option('wcce_options')['wcce_field_salesman_address_neighborhood'],
      ]
    ];
    $client   = [
      'name'     => $order->get_formatted_billing_full_name(),
      'phone'    => $order->get_billing_phone(),
      'document' => $order->get_meta('_billing_document'),
      'address'  => [
        'street'       => $order->get_billing_address_1() . ' ' . $order->get_billing_address_2(),
        'number'       => $order->get_meta('_billing_address_number'),
        'zipcode'      => $order->get_billing_postcode(),
        'neighborhood' => $order->get_meta('_billing_neighborhood'),
      ]
    ];
    $options  = [
      'extra_notify' => 'yes' === get_option('wcce_options')['wcce_field_receipt_notice'] ? 1 : 0,
      'extra_declared_value' => 'yes' === get_option('wcce_options')['wcce_field_declared_value'] ? $order->get_total() - $order->get_shipping_total() : ''
    ];

    switch ($shipping_method[0]) {
      case 'Correios':
        $postal['company_id'] = 1;
        break;

      case 'Jadlog':
        $postal['company_id'] = 2;
        break;
      
      default:
        $postal['company_id'] = 1;
        break;
    }

    $api->create_cart(
      $cart_name,
      $salesman,
      $client,
      $postal,
      $measurements,
      $options
    );
  }

  public static function activate()
  {
    update_option('rewrite_rules', '');
  }

  public static function deactivate()
  {
    flush_rewrite_rules();
  }

  public static function uninstall()
  {
    delete_option('wcce_options');
  }
}
