<?php

// Prevent data leaks.
defined('ABSPATH') || exit;

class WCCE_Api
{
  private string $baseurl;
  private string $token;

  public function __construct(string $token)
  {
    $this->baseurl = 'https://app.conectenvios.com.br/api/v1';
    $this->token   = $token;
  }

  protected function is_available($address, $measurements)
  {
    return empty($this->token) || empty($address['to']) || empty($address['from']) || empty($measurements['weight']) ? false : true;
  }

  public function simulate_shipping(array $address, array $postal, array $measurements, array $options = [])
  {
    if (! $this->is_available($address, $measurements)) {
      return [];
    }

    $request_url  = "$this->baseurl/package/shipping";
    $request_data = [
      'method'      => 'POST',
      'timeout'     => 45,
      'redirection' => 5,
      'httpversion' => '1.0',
      'blocking'    => true,
      'headers'     => array(
        'Authorization' => "Bearer $this->token",
      ),
      "body" => [
        'type'                => 'box',
        'width'               => $measurements['width'],
        'height'              => $measurements['height'],
        'length'              => $measurements['length'],
        'weight'              => $measurements['weight'],
        'addr_to_cep'         => $address['to'],
        'addr_from_cep'       => $address['from'],
        "postal_company_id"   => $postal['company_id'],
        "postal_service_name" => $postal['service_name'],
      ]
    ];

    if ( 0 < $options['extra_declared_value'] ) {
			$request_data['body']['extra_declared_value'] = $options['extra_declared_value'];
		}

    $response = wp_safe_remote_post( $request_url, $request_data );

    if (is_wp_error($response)) {
      return [];
    }

    return json_decode($response['body'])->data;
  }

  public function create_cart(
    string $cart_name, array $salesman, array $client, array $postal, array $measurements, array $options = []
  )
  {
    $request_url  = "$this->baseurl/cart";

    $request_data = [
      'method'      => 'POST',
      'timeout'     => 45,
      'redirection' => 5,
      'httpversion' => '1.0',
      'blocking'    => true,
      'headers'     => array(
        'Authorization' => "Bearer $this->token",
      ),
      "body" => [
        'package' => [
          [
            'type'                   => 'box',
            'name'                   => $cart_name,
            'width'                  => $measurements['width'],
            'height'                 => $measurements['height'],
            'length'                 => $measurements['length'],
            'weight'                 => $measurements['weight'],
            'extra_notify'           => $options['extra_notify'],
            'extra_declared_value'   => $options['extra_declared_value'],
            'addr_from_name'         => $salesman['name'],
            'addr_from_phone'        => $salesman['phone'],
            'addr_from_document'     => $salesman['document'],
            'addr_from_address'      => $salesman['address']['street'],
            'addr_from_number'       => $salesman['address']['number'],
            'addr_from_cep'          => $salesman['address']['zipcode'],
            'addr_from_neighborhood' => $salesman['address']['neighborhood'],
            'addr_to_name'           => $client['name'],
            'addr_to_phone'          => $client['phone'],
            'addr_to_document'       => $client['document'],
            'addr_to_address'        => $client['address']['street'],
            'addr_to_cep'            => $client['address']['zipcode'],
            'addr_to_number'         => $client['address']['number'],
            'addr_to_neighborhood'   => $client['address']['neighborhood'],
            "postal_company_id"      => $postal['company_id'],
            "postal_service_name"    => $postal['service_name'],
          ]
        ]
      ]
    ];

    return wp_safe_remote_post( $request_url, $request_data );
  }
}
