<?php

// Prevent data leaks.
defined('ABSPATH') || exit;

class WCCE_Calculator
{
  private int $package_width;
  private int $package_length;
  private int $package_height;
  private float $package_weight;

  public function __construct()
  {
    $this->package_width = 0;
    $this->package_length = 0;
    $this->package_height = 0;
    $this->package_weight = 0;
  }

  public function calculate_cart_measurements()
  {
    foreach (WC()->cart->get_cart() as $item => $values) {
      $product        = wc_get_product($values['data']->get_id());
      $product_qty    = $values['quantity'];

      $product_dimensions = [
        $product->get_width(),
        $product->get_length(),
        $product->get_height()
      ];

      sort($product_dimensions);
      list($product_height, $product_width, $product_length) = $product_dimensions;

      $this->package_height += ($product_height * $product_qty);
      $this->package_width   = ($product_width > $this->package_width) ? $product_width : $this->package_width;
      $this->package_length  = ($product_length > $this->package_length) ? $product_length : $this->package_length;
    }

    return [
      'width' => $this->package_width,
      'length' => $this->package_length,
      'height' => $this->package_height,
      'weight' => wc_get_weight(WC()->cart->get_cart_contents_weight(), 'g')
    ];
  }

  public function calculate_order_measurements($order_id)
  {
    $order = wc_get_order($order_id);

    foreach ($order->get_items() as $item) {
      if ('WC_Order_Item_Product' === get_class($item)) {
        $product        = wc_get_product($item->get_data()['product_id']);
        $product_qty    = $item->get_data()['quantity'];

        $product_dimensions = [
          $product->get_width(),
          $product->get_length(),
          $product->get_height()
        ];

        sort($product_dimensions);
        list($product_height, $product_width, $product_length) = $product_dimensions;

        $this->package_height += ($product_height * $product_qty);
        $this->package_weight += ($product->get_weight() * $product_qty);
        $this->package_width   = ($product_width > $this->package_width) ? $product_width : $this->package_width;
        $this->package_length  = ($product_length > $this->package_length) ? $product_length : $this->package_length;
      }
    }

    return [
      'width'  => intval($this->package_width),
      'length' => intval($this->package_length),
      'height' => intval($this->package_height),
      'weight' => wc_get_weight($this->package_weight, 'g')
    ];
  }
}
