<?php

// Prevent data leaks.
defined('ABSPATH') || exit;

abstract class WCCE_Shipping_Method extends WC_Shipping_Method
{
	private int $wcce_company_id;
	private string $wcce_service_name;

	public function __construct(int $instance_id = 0, int $wcce_company_id, string $wcce_service_name)
	{
		$this->enabled            = isset($this->settings['enabled']) ? $this->settings['enabled'] : 'yes';
		$this->instance_id        = absint($instance_id);
		$this->wcce_company_id    = $wcce_company_id;
		$this->wcce_service_name  = $wcce_service_name;
		$this->supports           = [
			'shipping-zones',
			'instance-settings',
			'instance-settings-modal',
		];

		$this->instance_form_fields = [
			'behavior_options'   => [
				'title'   => __('Opções de comportamento', 'woocommerce-conectenvios'),
				'type'    => 'title',
				'default' => '',
			],
			'additional_time'    => [
				'title'       => __('Dias adicionais', 'woocommerce-conectenvios'),
				'type'        => 'text',
				'description' => __('Dias úteis adicionais para a previsão de entrega.', 'woocommerce-conectenvios'),
				'desc_tip'    => true,
				'default'     => '0',
				'placeholder' => '0',
			],
			'fee'                => [
				'title'       => __('Taxa de manuseio', 'woocommerce-conectenvios'),
				'type'        => 'price',
				'description' => __('Insira um valor em R$. Deixe em branco para desativar.', 'woocommerce-conectenvios'),
				'desc_tip'    => true,
				'placeholder' => '0,00',
				'default'     => '',
			],
		];

		// Save settings in admin if you have any defined
		add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
	}

	public function calculate_shipping($package = [])
	{
		$token   = get_option('wcce_options')['wcce_field_token'];
		$address = [
			'to'   => $package['destination']['postcode'],
			'from' => get_option('wcce_options')['wcce_field_salesman_address_zipcode']
		];
		$postal  = [
			'company_id'   => $this->wcce_company_id,
			'service_name' => $this->wcce_service_name
		];
		$options = [
			'extra_notify'         => 'yes' === get_option('wcce_options')['wcce_field_receipt_notice'] ? 1 : 0,
			'extra_declared_value' => 'yes' === get_option('wcce_options')['wcce_field_declared_value'] ? floatval(WC()->cart->subtotal) : 0,
		];

		$api  = new WCCE_Api($token);
		$calc = new WCCE_Calculator();

		$measurements = $calc->calculate_cart_measurements();
		$simulation   = $api->simulate_shipping($address, $postal, $measurements, $options);

		if ([] === $simulation) {
			return null;
		}

		// $rate = array(
		// 	'id'    => $this->id,
		// 	'label' => $this->title,
		// 	'cost'  => $simulation[0]->price_discounted
		// );

		// Display delivery.
		$meta_delivery = array();
		if ('yes' === get_option('wcce_options')['wcce_field_show_delivery_time']) {
			$meta_delivery = [
				'_delivery_forecast' => intval($simulation[0]->deadline) + intval($this->get_instance_option('additional_time')),
			];
		}

		$rate = apply_filters(
			'woocommerce_' . $this->id . '_shipping_add_rate',
			array(
				'id'        => $this->get_rate_id(),
				'label'     => $this->title,
				'cost'      => (float) $simulation[0]->price_discounted + (float) $this->get_instance_option('fee'),
				'meta_data' => $meta_delivery,
			),
			$this->instance_id,
			$package
		);

		$this->add_rate($rate);
	}
}
